package cn.wise.sc.pay.common.security2.core;

import cn.hutool.core.util.StrUtil;
import cn.wise.sc.pay.common.security2.exception.JwtTokenExpirationException;
import cn.wise.sc.pay.common.security2.exception.JwtTokenInvalidException;
import cn.wise.sc.pay.common.security2.util.JwtUtils;
import io.jsonwebtoken.ExpiredJwtException;
import lombok.extern.slf4j.Slf4j;
import org.redisson.api.RedissonClient;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.security.web.authentication.WebAuthenticationDetailsSource;
import org.springframework.stereotype.Component;
import org.springframework.web.filter.OncePerRequestFilter;

import javax.servlet.FilterChain;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;

/**
 * @ClassNameCustomRequestFilter
 * @Description security的先决执行器，once只执行一次
 * 主要用于验证token,为当次请求加上UsernamePasswordAuthenticationToken信息
 * @Author qh
 * @Date2020/3/5 14:36
 * @Version V1.0
 **/
@Component
@Slf4j
public class JwtRequestFilter extends OncePerRequestFilter {

    private final UserDetailsService jwtUserDetailsService;

    private final RedissonClient redissonClient;

    @Autowired
    JdbcTemplate jdbcTemplate;

    public JwtRequestFilter(UserDetailsService jwtUserDetailsService, RedissonClient redissonClient) {
        this.jwtUserDetailsService = jwtUserDetailsService;
        this.redissonClient = redissonClient;
    }

    @Override
    protected void doFilterInternal(HttpServletRequest httpServletRequest, HttpServletResponse httpServletResponse, FilterChain filterChain) throws ServletException, IOException {

        String requestTokenHeader = httpServletRequest.getHeader(JwtUtils.TOKEN_HEADER);
        String username = null;
        String token = null;
        UserDetails sysUser = null;
        //如果token不为空 解析token
        if (requestTokenHeader != null && requestTokenHeader.startsWith(JwtUtils.TOKEN_PREFIX)) {
            token = requestTokenHeader.substring(7);
            //双层验证token,1.正常过期  2.人为操作过期(禁用)
            try {
                sysUser = JwtUtils.decodeToken(token);
            } catch (ExpiredJwtException e) {
                filterChain.doFilter(httpServletRequest, httpServletResponse);
                throw new JwtTokenExpirationException("登录已失效，请重新登录!");
            } catch (Exception e) {
                filterChain.doFilter(httpServletRequest, httpServletResponse);
                throw new JwtTokenInvalidException("请登录用户!");
            }
            username = sysUser.getUsername();
        }
        // 验证token信息 并存到SecurityContextHolder
        if (username != null && SecurityContextHolder.getContext().getAuthentication() == null) {
            try {
                UserDetails userDetails = this.jwtUserDetailsService.loadUserByUsername(username);

                //判断用户状态是否可用
                if (StrUtil.isEmpty(userDetails.getUsername()) || !userDetails.isEnabled()) {
                    filterChain.doFilter(httpServletRequest, httpServletResponse);
                    throw new JwtTokenInvalidException("用户已被注销 Σ(っ °Д °;)っ!");
                }
                if (!userDetails.isAccountNonLocked()) {
                    filterChain.doFilter(httpServletRequest, httpServletResponse);
                    throw new JwtTokenInvalidException("用户已被禁用,请联系管理员!");
                }
                if (!JwtUtils.isExpirationToken(token)) {
                    //为本次请求加上UsernamePasswordAuthenticationToken
                    //UsernamePasswordAuthenticationFilter会拿UsernamePasswordAuthenticationToken做授权验证
                    UsernamePasswordAuthenticationToken usernamePasswordAuthenticationToken = new UsernamePasswordAuthenticationToken(
                            userDetails, null, userDetails.getAuthorities());
                    usernamePasswordAuthenticationToken
                            .setDetails(new WebAuthenticationDetailsSource().buildDetails(httpServletRequest));
                    SecurityContextHolder.getContext().setAuthentication(usernamePasswordAuthenticationToken);
                }
            } catch (UsernameNotFoundException e) {
                filterChain.doFilter(httpServletRequest, httpServletResponse);
                throw new JwtTokenInvalidException("用户无效!");
            }
        }
        //此处放行 由后续UsernamePasswordAuthenticationToken做验证UsernamePasswordAuthenticationToken
        filterChain.doFilter(httpServletRequest, httpServletResponse);
    }
}

