package cn.wise.sc.pay.pass.business.app.controller.points;

import cn.wise.sc.pay.api.order.service.IOrdersService;
import cn.wise.sc.pay.api.points.service.IGoodRecordService;
import cn.wise.sc.pay.api.points.service.IMemberShipService;
import cn.wise.sc.pay.api.points.service.IPointDetailService;
import cn.wise.sc.pay.api.points.service.IPointPhotoService;
import cn.wise.sc.pay.api.points.service.IPointsSourceV1Service;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.common.core.query.PageQuery;
import cn.wise.sc.pay.domain.order.model.OrderResponse;
import cn.wise.sc.pay.domain.order.model.PayStatus;
import cn.wise.sc.pay.domain.point.bean.PointPhoto;
import cn.wise.sc.pay.domain.point.query.GoodExchangeQuery;
import cn.wise.sc.pay.domain.point.query.GoodRecordAddressQuery;
import cn.wise.sc.pay.domain.point.query.PointDetailQuery;
import cn.wise.sc.pay.domain.point.vo.UserExtraVOApp;
import com.alibaba.fastjson.JSON;
import io.swagger.annotations.ApiOperation;
import org.springframework.cloud.openfeign.SpringQueryMap;
import org.springframework.http.ResponseEntity;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import java.util.HashMap;
import java.util.Map;

/**
 * @description: 积分控制器
 * @author: qh
 * @create: 2020-04-28 15:47
 **/
@Validated
@RestController
@CrossOrigin
@RequestMapping("/points")
public class PointsController {

    final
    IGoodRecordService iGoodRecordService;
    final
    IMemberShipService iMemberShipService;
    final
    IPointDetailService iPointDetailService;
    final
    IPointsSourceV1Service iPointsSourceV1Service;
    final
    IPointPhotoService iPointPhotoService;
    final
    IOrdersService iOrdersService;

    public PointsController(IGoodRecordService iGoodRecordService,
                            IMemberShipService iMemberShipService,
                            IPointDetailService iPointDetailService,
                            IPointsSourceV1Service iPointsSourceV1Service,
                            IPointPhotoService iPointPhotoService,
                            IOrdersService iOrdersService) {
        this.iGoodRecordService = iGoodRecordService;
        this.iMemberShipService = iMemberShipService;
        this.iPointDetailService = iPointDetailService;
        this.iPointsSourceV1Service = iPointsSourceV1Service;
        this.iPointPhotoService = iPointPhotoService;
        this.iOrdersService = iOrdersService;
    }

    /**
     * 根据id获取用户兑换记录
     *
     * @param userId    用户id
     * @param pageQuery 分页对象
     * @return
     */
    @GetMapping("/page/{userId}")
    @ApiOperation("根据id获取用户兑换记录")
    BaseResponse pageByUserId(@PathVariable("userId") Long userId,
                              @SpringQueryMap PageQuery pageQuery) {
        return iGoodRecordService.pageByUserId(userId, pageQuery);
    }

    /**
     * 设置地址
     *
     * @param goodRecordAddressQuery
     * @return
     */
    @PostMapping("/add/address")
    @ApiOperation("设置地址")
    BaseResponse addAddressById(@RequestBody GoodRecordAddressQuery goodRecordAddressQuery) {
        return iGoodRecordService.addAddressById(goodRecordAddressQuery);
    }

    /**
     * 商品兑换
     *
     * @param goodExchangeQuery
     * @return
     */
    @PostMapping("/exchangeGoods")
    @ApiOperation("商品兑换")
    BaseResponse exchangeGoods(@RequestBody GoodExchangeQuery goodExchangeQuery) {
        return iGoodRecordService.exchangeGoods(goodExchangeQuery);
    }

    /**
     * 分页显示积分可兑换商品列表
     *
     * @param pageQuery
     * @return
     */
    @GetMapping("/exchangeRecord")
    @ApiOperation("分页显示积分可兑换商品列表")
    BaseResponse pageExchange(@SpringQueryMap PageQuery pageQuery) {
        return iGoodRecordService.pageExchange(pageQuery);
    }

    /**
     * 获取最新的兑换记录
     *
     * @param size
     * @return
     */
    @GetMapping("/latestRecords/{size}")
    @ApiOperation("获取最新的兑换记录")
    BaseResponse getLatestRecords(@PathVariable("size") Integer size) {
        return iGoodRecordService.getLatestRecords(size);
    }

    /**
     * 根据会员id获取会员信息以及会员权益信息
     *
     * @param memberId
     * @return
     */
    @GetMapping("/getMemberInfo")
    @ApiOperation("根据会员id获取会员信息以及会员权益信息")
    BaseResponse<UserExtraVOApp> getMemberInfo(Long memberId) {

        return iMemberShipService.getMemberInfo(memberId);
    }

    @PostMapping("/add")
    @ApiOperation("添加积分详情")
    ResponseEntity add(@RequestBody PointDetailQuery pointDetailQuery) {
        return iPointDetailService.add(pointDetailQuery);
    }

    /**
     * 查询用户积分消费记录
     *
     * @param pageNo   起始页
     * @param pageSize 每页大小
     * @param userId   用户id
     * @return
     */
    @GetMapping("/consume/{userId}")
    @ApiOperation("查询用户积分消费记录")
    BaseResponse listByConsume(@RequestParam("pageNo") Integer pageNo,
                               @RequestParam("pageSize") Integer pageSize,
                               @PathVariable("userId") Long userId) {
        return iPointDetailService.listByConsume(pageNo, pageSize, userId);
    }

    /**
     * 查询用户积分增加记录
     *
     * @param pageNo   起始页
     * @param pageSize 每页大小
     * @param userId   用户id
     * @return
     */
    @GetMapping("/increase/{userId}")
    @ApiOperation("查询用户积分增加记录")
    BaseResponse listByIncrease(@RequestParam("pageNo") Integer pageNo,
                                @RequestParam("pageSize") Integer pageSize,
                                @PathVariable("userId") Long userId) {
        return iPointDetailService.listByIncrease(pageNo, pageSize, userId);
    }

    /**
     * 根据途径查询和数据列表查询
     *
     * @param way
     * @return
     */
    @GetMapping(value = "/list/condition")
    @ApiOperation("根据途径查询和数据列表查询")
    BaseResponse listOnCondition(@RequestParam("way") String way) {
        return iPointsSourceV1Service.listOnCondition(way);
    }

    @PostMapping("/pointPhoto")
    public BaseResponse create(@RequestBody @Valid PointPhoto model) {

        //验证订单
        Map<String, Object> condition = new HashMap<>(1);
        condition.put("order_no", model.getOrderNo());
        condition.put("user_id", model.getUserId());
        condition.put("pay_status", PayStatus.PAY_YES.name());
        OrderResponse oneOrder = iOrdersService.getOneOrder(JSON.toJSONString(condition));
        if (oneOrder.getCode() != 200 || oneOrder.getData() == null) {
            return BaseResponse.error("订单号无效!", this.getClass().getName());
        }
        return iPointPhotoService.create(model);
    }

}
