package cn.wise.sc.pay.business.association.service.impl;

import cn.wise.sc.pay.business.association.configure.FastDFSUtils;
import cn.wise.sc.pay.business.association.mapper.ArticleMapper;
import cn.wise.sc.pay.business.association.service.IArticleBannerService;
import cn.wise.sc.pay.business.association.service.IArticleService;
import cn.wise.sc.pay.business.association.service.IArticleSourceService;
import cn.wise.sc.pay.business.association.service.IBannerService;
import cn.wise.sc.pay.business.association.service.IBlacklistService;
import cn.wise.sc.pay.business.association.service.ICollectionService;
import cn.wise.sc.pay.business.association.service.IPraiseService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.common.core.query.PageQuery;
import cn.wise.sc.pay.common.core.query.TimeBucket;
import cn.wise.sc.pay.common.core.util.DateTimeUtils;
import cn.wise.sc.pay.common.core.util.FileTypeUtils;
import cn.wise.sc.pay.domain.association.bean.Article;
import cn.wise.sc.pay.domain.association.bean.ArticleBanner;
import cn.wise.sc.pay.domain.association.bean.ArticleSource;
import cn.wise.sc.pay.domain.association.bean.Banner;
import cn.wise.sc.pay.domain.association.bean.Blacklist;
import cn.wise.sc.pay.domain.association.bean.Collection;
import cn.wise.sc.pay.domain.association.bean.Praise;
import cn.wise.sc.pay.domain.association.query.ArticleQuery;
import cn.wise.sc.pay.domain.association.vo.ArticleVO;
import cn.wise.sc.pay.domain.association.vo.AuthorArticleCountVO;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.OrderItem;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.io.FilenameUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 服务实现类
 *
 * @author lqk
 * @since 2019-11-19
 */
@Service
@Slf4j
public class ArticleServiceImpl extends ServiceImpl<ArticleMapper, Article>
        implements IArticleService {

    @Autowired
    ArticleMapper articleMapper;
    @Autowired
    IArticleSourceService sourceService;
    @Autowired
    IBlacklistService blacklistService;
    @Autowired
    IArticleBannerService articleBannerService;
    @Autowired
    IBannerService bannerService;
    @Autowired
    FastDFSUtils fastDFSUtils;
    @Autowired
    ICollectionService collectionService;
    @Autowired
    IPraiseService praiseService;
    @Autowired
    Producer producer;

    @Override
    public BaseResponse addArticle(ArticleQuery articleQuery) {

        // 判断发帖者是否在黑名单上
        if (articleQuery == null) {
            return BaseResponse.error("发帖失败", Article.class);
        }
        QueryWrapper<Blacklist> wrapper = new QueryWrapper<>();
        wrapper.eq("user_id", articleQuery.getAuthorId());
        List<Blacklist> lists = blacklistService.list(wrapper);
        if (lists != null && lists.size() > 0) {
            return BaseResponse.error("账号在发帖黑名单，请联系管理员", Article.class);
        }
        String bname = articleQuery.getBname();
        if (StringUtils.isBlank(bname)) {
            return BaseResponse.error("请选择图文/文章分类", Article.class);
        }
        // 添加文章
        Article article =
                Article.builder()
                        .authorId(articleQuery.getAuthorId())
                        .authorName(articleQuery.getAuthorName())
                        .authorImage(articleQuery.getAuthorImage())
                        .picType(articleQuery.getPicType())
                        .address(articleQuery.getAddress())
                        .title(articleQuery.getTitle())
                        .content(articleQuery.getContent())
                        .coverImage(articleQuery.getCoverImage())
                        .createTime(System.currentTimeMillis())
                        .updateTime(System.currentTimeMillis())
                        .type(articleQuery.getType())
                        .build();

        // 判断是否需要审核
        if (articleQuery.getStatus() != 0) {
            article.setStatus((byte) 1);
        }

        this.save(article);

        // 添加文章分类
        List<ArticleBanner> articleBanners = new ArrayList<>();
        if (StringUtils.isNotEmpty(bname)) {
            String[] types = bname.split(",");
            for (String type : types) {
                ArticleBanner articleBanner =
                        ArticleBanner.builder()
                                .articleId(article.getId())
                                .bannerId(Integer.parseInt(type))
                                .build();
                articleBanners.add(articleBanner);
            }
            if (articleBanners.size() > 0) {
                articleBannerService.saveBatch(articleBanners);
            }
        }

        // 添加文章资源
        List<String> sourceList = articleQuery.getAocSources();
        List<ArticleSource> aocArticleSources = new ArrayList<>();
        if (sourceList != null && sourceList.size() > 0) {
            int order = 1;
            for (String source : sourceList) {
                int type = FileTypeUtils.checkFileExtension(FilenameUtils.getExtension(source));
                ArticleSource aocArticleSource =
                        ArticleSource.builder()
                                .articleId(article.getId())
                                .url(source)
                                .orderNum(order)
                                .type(type)
                                .build();
                aocArticleSources.add(aocArticleSource);
                order++;
            }
            if (aocArticleSources.size() > 0) {
                sourceService.saveBatch(aocArticleSources);
            }
        }

        //自动审核开启时==>推送积分
        if (article.getStatus() == 1) {
            sendArticlePoints(article);
        }

        return BaseResponse.ok("发布成功");
    }

    @Override
    public BaseResponse<Page<Article>> articlesNotVerify(
            String authorName, TimeBucket timeBucket, PageQuery pageQuery) {
        Page page = new Page(pageQuery.getPageNo(), pageQuery.getPageSize());
        QueryWrapper wrapper = new QueryWrapper();
        // 修改添加排序字段
        wrapper.orderByDesc("create_time", "update_time");
        //        boolean flag = true;
        if (DateTimeUtils.validTimeBucket(timeBucket)) {
            wrapper.between("create_time", timeBucket.getStartTime(), timeBucket.getEndTime());
            //            flag = false;
        }
        if (StringUtils.isNotEmpty(authorName)) {
            wrapper.like("author_name", authorName);
            //            flag = false;
        }
        //        if (flag) wrapper = null;
        wrapper.isNotNull("author_name");
        List<Article> list = articleMapper.articleList(page, wrapper, null, (byte) 0);
        if (list == null || list.size() == 0) {
            return BaseResponse.ok(page);
        }
        List<Long> userIds = new ArrayList<>();
        list.stream()
                .forEach(
                        arg -> {
                            userIds.add(arg.getAuthorId());
                        });
        List<AuthorArticleCountVO> articleCounts = articleMapper.selectUserArticles(userIds);
        list.stream()
                .forEach(
                        arg -> {
                            arg.setCoverImage(fastDFSUtils.fullUrl(arg.getCoverImage()));
                            arg.setSummary(fastDFSUtils.getSummary(arg.getContent()));
                            articleCounts.stream()
                                    .forEach(
                                            para -> {
                                                if (arg.getAuthorId().longValue() == para.getAuthorId().longValue()) {
                                                    arg.setArticleNum(para.getNum());
                                                }
                                            });
                        });

        page.setRecords(list);
        return BaseResponse.ok(page);
    }

    @Override
    public BaseResponse updateArticleStatus(List<Long> ids, byte status) {

        if (ids != null && ids.size() > 0) {
            articleMapper.updateArticleStatus(ids, status);
            QueryWrapper<Article> queryWrapper = new QueryWrapper<>();
            queryWrapper.in("id", ids);
            List<Article> articles = this.list(queryWrapper);

            //为审核通过的文章推送积分增加消息
            for (Article article : articles) {
                if (article.getStatus() == 1) {
                    sendArticlePoints(article);
                }
            }

            return BaseResponse.ok("修改成功");
        } else {
            return BaseResponse.error("批量修改失败", Article.class);
        }
    }


    @Override
    public BaseResponse<Page<Article>> articlesVerify(
            Integer type, String order, String name, String title, PageQuery pageQuery) {

        Page<Article> page = new Page<>();
        page.setSize(pageQuery.getPageSize());
        //        page.setAsc("create_time");
        page.setCurrent(pageQuery.getPageNo());

        QueryWrapper wrapper = new QueryWrapper();
        // 升序
        wrapper.isNotNull("a.id");
        if (StringUtils.isNotEmpty(title)) {
            wrapper.like("title", title);
        }
        if (StringUtils.isNotEmpty(name)) {
            wrapper.like("author_name", name);
        }
        if (StringUtils.isNotEmpty(order)) {
            if ("asc".equalsIgnoreCase(order)) {
                wrapper.orderByAsc("comnum");
            } else if ("desc".equalsIgnoreCase(order)) {
                wrapper.orderByDesc("comnum");
            } else {
                wrapper.orderByDesc("createTime");
            }
        } else {
            wrapper.orderByDesc("create_time", "update_time");
        }
        List<Article> list = articleMapper.articleList(page, wrapper, type, (byte) 1);
        for (Article article : list) {
            article.setCoverImage(fastDFSUtils.fullUrl(article.getCoverImage()));
            article.setSummary(fastDFSUtils.getSummary(article.getContent()));
        }
        page.setRecords(list);
        return BaseResponse.ok(page);
    }

    @Override
    public BaseResponse deleteArticlesById(List<Long> idList) {
        if (idList == null || idList.size() == 0) {
            return BaseResponse.error("请选择需要删除的数据", Article.class);
        }
        UpdateWrapper<Article> wrapper = new UpdateWrapper<>();
        wrapper.in("id", idList);
        articleMapper.deleteArticles(idList);
        return BaseResponse.ok("数据已删除");
        //        //文章评论列表
        //        EntityWrapper<Comment> entityWrapper = new EntityWrapper<>();
        //        entityWrapper.in("article_id", idList);
        //        List<Comment> commentList = articleCommentService.selectList(entityWrapper);
        //
        //        //删除文章/图文,评论
        //        this.deleteBatchIds(idList);
        //        articleCommentService.deleteByTargetIds(idList);
        //
        //        //删除评论回复
        //        List<Long> commentIds = new ArrayList<>();
        //        for (Comment comment : commentList) {
        //            commentIds.add(comment.getId());
        //        }
        //        if (commentIds.size() > 0) {
        //            commentReplyService.deleteByCommentId(commentIds);
        //        }
        //        // 文章/图文中的图片等资源
        //        EntityWrapper<ArticleSource> wrapper = new EntityWrapper();
        //        wrapper.in("aoc_id", idList);
        //        wrapper.eq("type", 0);
        //        List<ArticleSource> sources = sourceMapper.selectList(wrapper);
        //        //评论中的图片等资源
        //        wrapper = new EntityWrapper<>();
        //        wrapper.in("aoc_id", commentIds);
        //        wrapper.eq("type", 1);
        //        sources.addAll(sourceMapper.selectList(wrapper));
        //
        //        //删除文章/图文资源
        //        for (ArticleSource aocSource : sources) {
        //            FastDFSUtils.delete(aocSource.getUrl());
        //        }
    }

    @Override
    public BaseResponse uploadPic(MultipartFile file) {
        Map<String, String> map = new HashMap<>();
        map.put("host", fastDFSUtils.fullUrl(""));
        String path = "";
        if (file != null && StringUtils.isNotEmpty(file.getOriginalFilename())) {
            try {
                path = fastDFSUtils.uploadFile(file).getPath();
            } catch (IOException e) {
                return BaseResponse.error("上传失败", MultipartFile.class);
            }
        }
        map.put("uri", path);
        return BaseResponse.ok(map);
    }

    @Override
    public BaseResponse selectArticleInfo(Long id) {
        if (id == null) {
            return BaseResponse.error("参数错误", Article.class);
        }
        // 图文表内容
        Article article = articleMapper.selectById(id);
        if (article == null) {
            return BaseResponse.ok(null);
        }
        // 图文资源表内容
        QueryWrapper<ArticleSource> wrapper = new QueryWrapper<>();
        wrapper.eq("article_id", id);
        List<ArticleSource> list = sourceService.list(wrapper);
        for (ArticleSource source : list) {
            source.setUrl(fastDFSUtils.fullUrl(source.getUrl()));
        }
        article.setArticleSources(list);
        article.setCoverImage(fastDFSUtils.fullUrl(article.getCoverImage()));
        // 所属分类字符串
        QueryWrapper<ArticleBanner> bWrapper = new QueryWrapper<>();
        bWrapper.eq("article_id", id);
        List<ArticleBanner> abList = articleBannerService.list(bWrapper);
        List<Integer> abIds = new ArrayList<>();
        for (ArticleBanner ab : abList) {
            abIds.add(ab.getBannerId());
        }
        if (abIds != null && abIds.size() > 0) {
            QueryWrapper<Banner> bannerWrapper = new QueryWrapper<>();
            bannerWrapper.in("id", abIds);
            List<Banner> banners = bannerService.list(bannerWrapper);
            StringBuilder bname = new StringBuilder();
            for (Banner ba : banners) {
                bname.append(ba.getName() + ",");
            }
            article.setBname(bname.substring(0, bname.length() - 1));
        }
        return BaseResponse.ok(new ArticleVO(article));
    }

    @Override
    public boolean updateArticleComNum(Long targetId) {
        Article article = this.getById(targetId);
        article.setComNum(article.getComNum() + 1);
        return this.updateById(article);
    }

    @Override
    public BaseResponse<Page<Article>> getCollectionArticle(Long userId, PageQuery pageQuery) {
        if (userId == null) {
            return null;
        }
        QueryWrapper<Collection> coWrapper = new QueryWrapper<>();
        coWrapper.eq("user_id", userId);
        List<Collection> list = collectionService.list(coWrapper);
        List<Long> ids = new ArrayList<>();
        list.stream()
                .forEach(
                        arg -> {
                            ids.add(arg.getArticleId());
                        });
        Page page = new Page<>();
        page.setCurrent(pageQuery.getPageNo());
        page.setSize(pageQuery.getPageSize());
        if (ids == null || ids.size() == 0) {
            return BaseResponse.ok(page);
        }
        QueryWrapper<Article> wrapper = new QueryWrapper<>();
        wrapper.in("id", ids);
        page = (Page) this.page(page, wrapper);
        List<Article> orders = page.getRecords();
        orders.stream()
                .forEach(
                        arg -> {
                            arg.setCoverImage(fastDFSUtils.fullUrl(arg.getCoverImage()));
                            arg.setSummary(fastDFSUtils.getSummary(arg.getContent()));
                        });
        page.setRecords(orders);
        return BaseResponse.ok(page);
    }

    @Override
    public BaseResponse updateArticleViewNum(Long id) {
        if (id == null) {
            return BaseResponse.error("参数有误", Article.class);
        }
        Article article = this.getById(id);
        article.setViewNum(article.getViewNum() + 1);
        Boolean result = this.updateById(article);
        return result ? BaseResponse.ok("ok") : BaseResponse.error("fail", Article.class);
    }

    @Override
    public BaseResponse<Page<Article>> getUserPublishArticle(Long userId, PageQuery pageQuery) {
        if (userId != null) {
            Page page = new Page<>();
            page.setSize(pageQuery.getPageSize());
            page.setCurrent(pageQuery.getPageNo());
            page.addOrder(OrderItem.desc("create_time"));
            QueryWrapper<Article> wrapper = new QueryWrapper<>();
            wrapper.eq("author_id", userId);
            wrapper.eq("del_status", 0);
            page = (Page) this.page(page, wrapper);
            List<Article> articles = page.getRecords();
            articles.stream()
                    .forEach(
                            arg -> {
                                arg.setCoverImage(fastDFSUtils.fullUrl(arg.getCoverImage()));
                                arg.setSummary(fastDFSUtils.getSummary(arg.getContent()));
                            });
            page.setRecords(articles);
            return BaseResponse.ok(page);
        }
        return null;
    }

    @Override
    public BaseResponse<Page<Article>> getUserPraiseArticle(Long userId, PageQuery pageQuery) {
        if (userId != null) {
            QueryWrapper<Praise> coWrapper = new QueryWrapper<>();
            coWrapper.eq("user_id", userId);
            List<Praise> list = praiseService.list(coWrapper);
            List<Long> ids = new ArrayList<>();
            for (Praise praise : list) {
                ids.add(praise.getArticleId());
            }
            Page page = new Page<>();
            page.setCurrent(pageQuery.getPageNo());
            page.setSize(pageQuery.getPageSize());
            QueryWrapper<Article> wrapper = new QueryWrapper<>();
            wrapper.in("id", ids);
            if (ids == null || ids.size() == 0) {
                return BaseResponse.ok(page);
            }
            page = (Page) this.page(page, wrapper);
            List<Article> articles = page.getRecords();
            articles.stream()
                    .forEach(
                            arg -> {
                                arg.setCoverImage(fastDFSUtils.fullUrl(arg.getCoverImage()));
                                arg.setSummary(fastDFSUtils.getSummary(arg.getContent()));
                            });
            page.setRecords(articles);
            return BaseResponse.ok(page);
        }
        return null;
    }

    @Override
    public Integer getMyPraiseSum(Long userId) {
        return articleMapper.getPraiseSum(userId);
    }

    @Override
    public BaseResponse preOrNextArticle(Long articleId, Integer pon, Integer status) {
        if (articleId == null || pon == null) {
            return BaseResponse.error("参数错误", Article.class);
        }
        QueryWrapper wrapper = new QueryWrapper();
        if (pon == 0) {
            wrapper.lt("id", articleId);
            wrapper.orderByDesc("id");

        } else {
            wrapper.gt("id", articleId);
            wrapper.orderByAsc("id");
        }
        if (status == 1) {
            wrapper.eq("status", 1);
        } else {
            wrapper.eq("status", 0);
        }
        wrapper.last("limit 1 ");
        Article article = this.getOne(wrapper);
        if (article != null) {
            article.setCoverImage(fastDFSUtils.fullUrl(article.getCoverImage()));
            return BaseResponse.ok(article);
        }
        return BaseResponse.error(pon == 0 ? "已到第一篇" : "已到最后一篇", Article.class);
    }


    /**
     * 文章推送积分
     *
     * @param article 待推送的文章
     */
    private void sendArticlePoints(Article article) {
        // 初始化队列
        try {
            log.info("=======================文章审核通过=============================");
            producer.createQueue("pointArticleAndExpQue");
        } catch (IOException e) {
            log.error("文章审核通过,获取推送积分失败!");
        }

        // 修改成功后推文章积分mq
        producer.send("pointArticleAndExpQue", JSON.toJSONString(article));
    }
}
