package cn.wise.sc.pay.api.cashier.service;

import cn.wise.sc.pay.api.cashier.service.impl.GoodsServiceImplHystrix;
import cn.wise.sc.pay.api.cashier.service.impl.GoodsSpecExcel;
import cn.wise.sc.pay.business.goods.model.GoodsResponse;
import cn.wise.sc.pay.business.goods.model.bean.Goods;
import cn.wise.sc.pay.business.goods.model.query.GoodsQuery;
import cn.wise.sc.pay.business.goods.model.query.GoodsScanQuery;
import cn.wise.sc.pay.business.goods.model.query.GoodsTypeQuery;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.common.core.query.PageQuery;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;
import java.util.List;

/**
 * @author qh
 */
@FeignClient(value = "saas-v1-cashier", path = "/goods", fallback = GoodsServiceImplHystrix.class)
public interface IGoodsService {

    /**
     * 所有商品集合
     *
     * @param sellerId 商户id
     * @param key      关键字
     * @param type     类型
     * @param status   状态 0下架 1上架
     * @return BaseResponse
     */
    @GetMapping("/list/{sellerId}")
    BaseResponse list(@PathVariable("sellerId") int sellerId, @RequestParam("key") String key,
                      @RequestParam("type") String type, @RequestParam("status") int status);

    /**
     * 按照类型获取list
     *
     * @param key
     * @param type   类型
     * @param status 状态 0下架 1上架 -1:全部
     * @return BaseResponse
     */
    @GetMapping("/list/user")
    BaseResponse listUser(@RequestParam("key") String key, @RequestParam("type") String type, @RequestParam("status") int status);

    /**
     * 获取商品规格
     *
     * @param goodsId 商品id
     * @return BaseResponse
     */
    @GetMapping("/spc/{goodsId}")
    BaseResponse getGoodsSpc(@PathVariable("goodsId") int goodsId);

    /**
     * 获取商品
     *
     * @param goodsId 商品id
     * @return BaseResponse
     */
    @GetMapping("/{goodsId}")
    BaseResponse getGoods(@PathVariable("goodsId") int goodsId);

    /**
     * 添加一个商品
     *
     * @param goodsQuery 商品
     * @return BaseResponse
     */
    @PostMapping
    BaseResponse newGoods(@RequestBody GoodsQuery goodsQuery);

    /**
     * 编辑商品
     *
     * @param goodsQuery 商品
     * @return BaseResponse
     */
    @PutMapping
    BaseResponse edit(@RequestBody GoodsQuery goodsQuery);

    /**
     * 删除商品
     *
     * @param goodsId  商户id
     * @param sellerId 商品id
     * @return BaseResponse
     */
    @DeleteMapping("/{goodsId}/{sellerId}")
    BaseResponse delete(@PathVariable("goodsId") int goodsId, @PathVariable("sellerId") int sellerId);

    /**
     * 删除商品一级规格
     *
     * @param spcId    商户id
     * @param sellerId 一级规格id
     * @return BaseResponse
     */
    @DeleteMapping("/spc/{spcId}/{sellerId}")
    BaseResponse deleteSpc(@PathVariable("spcId") int spcId, @PathVariable("sellerId") int sellerId);

    /**
     * 删除商品二级规格
     *
     * @param detailId 商户id
     * @param sellerId 二级规格id
     * @return BaseResponse
     */
    @DeleteMapping("/spc/detail/{detailId}/{sellerId}")
    BaseResponse deleteSpcDetal(@PathVariable("detailId") int detailId, @PathVariable("sellerId") int sellerId);

    /**
     * 上架
     *
     * @param goodsId  商户id
     * @param sellerId 商品id
     * @return BaseResponse
     */
    @PutMapping("/status/1/{goodsId}/{sellerId}")
    BaseResponse shelveGoods(@PathVariable("goodsId") int goodsId, @PathVariable("sellerId") int sellerId);

    /**
     * 下架
     *
     * @param goodsId  商品id
     * @param sellerId 商户id
     * @return BaseResponse
     */
    @PutMapping("/status/0/{goodsId}/{sellerId}")
    BaseResponse unShelveGoods(@PathVariable("goodsId") int goodsId, @PathVariable("sellerId") int sellerId);

    /**
     * 下架(非主观下架,带原因
     *
     * @param goodsId  商户id
     * @param sellerId 商品id
     * @param cause    原因
     * @return BaseResponse
     */
    @PutMapping("/manage/status/0/{goodsId}/{sellerId}")
    GoodsResponse unShelveGoods(@PathVariable("goodsId") int goodsId, @PathVariable("sellerId") int sellerId,
                                @RequestParam("cause") String cause);

    /**
     * 获取待审核的商品集合
     *
     * @param pageQuery 商品名
     * @param key       商铺名
     * @param shopId
     * @param goodsType 商品类型
     * @return BaseResponse
     */
    @PostMapping("/list/waitAudit")
    BaseResponse getWaitingAuditList(@RequestBody PageQuery pageQuery, @RequestParam("key") String key, @RequestParam("shopId") Long shopId,
                                     @RequestParam("goodsType") String goodsType);

    /**
     * audit
     *
     * @param goodsId 商户id
     * @param status  状态  1:通过   3:不通过
     * @param cause   原因
     * @return BaseResponse
     */
    @PutMapping("/audit")
    BaseResponse auditGoods(@RequestParam("goodsId") long goodsId, @RequestParam("status") int status, @RequestParam("cause") String cause);

    /**
     * 获取所有的商品集合
     *
     * @param pageQuery
     * @param key       商品名
     * @param shopId    商铺id
     * @param goodsType 商品类型
     * @return BaseResponse
     */
    @PostMapping("/list")
    BaseResponse list(@RequestBody PageQuery pageQuery, @RequestParam("key") String key, @RequestParam("shopId") long shopId,
                      @RequestParam("goodsType") String goodsType);

    /**
     * 新增一个类型
     *
     * @param typeQuery 商品类型
     * @return BaseResponse
     */
    @PostMapping("/type")
    BaseResponse newGoodsType(@RequestBody GoodsTypeQuery typeQuery);

    /**
     * 获取类型集合
     *
     * @param sellerId 商户id
     * @param status   状态
     * @return BaseResponse
     */
    @GetMapping("/type/list/{sellerId}")
    BaseResponse typeList(@PathVariable("sellerId") int sellerId, @RequestParam("status") short status);

    /**
     * 编辑一个类型
     *
     * @param typeQuery 商品类型
     * @return BaseResponse
     */
    @PutMapping("/type")
    BaseResponse editType(@RequestBody GoodsTypeQuery typeQuery);

    /**
     * 删除一个类型
     *
     * @param id       类型id
     * @param sellerId 商户id
     * @return BaseResponse
     */
    @DeleteMapping("/type/{id}/{sellerId}")
    BaseResponse delType(@PathVariable("id") int id, @PathVariable("sellerId") int sellerId);

    /**
     * 删除一个类型以及下架该类别下的所有商品
     *
     * @param id       类型id
     * @param sellerId 商户id
     * @return BaseResponse
     */
    @DeleteMapping("/typeAndUnShelveGoods/{id}/{sellerId}")
    BaseResponse delTypeAndUnShelveGoods(@PathVariable("id") int id, @PathVariable("sellerId") int sellerId);


    /**
     * 扫码录入商品
     * @param goodsScanQuery 扫码商品
     * @return BaseResponse
     */
    @PostMapping("/scan")
    @ApiOperation("扫码录入商品")
    @ApiImplicitParam(value = "扫码商品", name = "goodsScanQuery", paramType = "body", dataTypeClass = GoodsScanQuery.class)
    BaseResponse<Goods> scanAddGoods(@RequestBody GoodsScanQuery goodsScanQuery);

    /**
     * 扫码获取商品信息
     * @param code  二维码
     * @return BaseResponse
     */
    @GetMapping("/scan")
    @ApiOperation("扫码获取商品信息")
    @ApiImplicitParam(name = "code", value = "二维码", paramType = "query", dataTypeClass = String.class)
    BaseResponse<Goods> getGoodsInfoByCode(@RequestParam("code") String code);

    /**
     * 批量添加
     * @param data List<GoodsSpecExcel>
     * @return BaseResponse
     */
    @PostMapping("/batch/add")
    BaseResponse<Goods> batchAdd(List<GoodsSpecExcel> data);

    /**
     * 获取商品所有规格
     *
     * @return List
     */
    @GetMapping("/allGoodsSpc")
    List<GoodsSpecExcel> getAllGoodsSpc();

    /**
     * 获取商品列表  根据ids
     * @param ids id集合
     * @return List
     */
    @PostMapping("/goodIds")
    List<Goods> getByGoodIds(@RequestBody List<Long> ids);
}

