package cn.wise.sc.pay.business.exhibition.service.impl;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.CharsetUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.crypto.SecureUtil;
import cn.hutool.crypto.symmetric.SymmetricAlgorithm;
import cn.hutool.crypto.symmetric.SymmetricCrypto;
import cn.wise.sc.pay.api.order.service.IOrdersService;
import cn.wise.sc.pay.business.exhibition.configure.FastDFSUtils;
import cn.wise.sc.pay.business.exhibition.configure.QRCodeProperties;
import cn.wise.sc.pay.business.exhibition.mapper.EntranceQrcodeMapper;
import cn.wise.sc.pay.business.exhibition.service.IEntranceLogService;
import cn.wise.sc.pay.business.exhibition.service.IEntranceQRcodeService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.domain.exhibition.bean.EntranceLog;
import cn.wise.sc.pay.domain.exhibition.bean.EntranceQrcode;
import cn.wise.sc.pay.domain.exhibition.po.Commodity;
import cn.wise.sc.pay.domain.exhibition.po.RQCodeInfoDo;
import cn.wise.sc.pay.domain.exhibition.query.QRCodeGiftQuery;
import cn.wise.sc.pay.domain.exhibition.vo.RQCodeInfoVo;
import cn.wise.sc.pay.domain.order.model.Orders;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.context.properties.EnableConfigurationProperties;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * <p>
 * 入场二维码 服务实现类
 * </p>
 *
 * @author qh
 * @since 2019-11-21
 */
@Service
@EnableConfigurationProperties(QRCodeProperties.class)
public class EntranceQrcodeServiceImpl extends ServiceImpl<EntranceQrcodeMapper, EntranceQrcode> implements IEntranceQRcodeService {

    @Autowired
    IOrdersService iOrdersService;
    @Autowired
    QRCodeProperties qrCodeProperties;
    @Autowired
    IEntranceLogService iEntranceLogService;
    @Autowired
    EntranceQrcodeMapper entranceQrcodeMapper;
    @Autowired
    FastDFSUtils fastDFSUtils;

    /**
     * 对称加密的密钥
     */
    private final byte[] key = SecureUtil.generateKey(SymmetricAlgorithm.AES.getValue()).getEncoded();

    @Override
    public BaseResponse getQRCode(Orders order) {
        if (order == null) {
            return BaseResponse.error("参数有误!", new EntranceQrcode());
        }
        List<Commodity> commoditys = JSON.parseArray(order.getGoods(), Commodity.class);
        //为每张票构建一个入场二维码
        List<EntranceQrcode> entranceQrcodesInsert = new ArrayList<>();
        for (Commodity commodity : commoditys) {
            for (int i = 0; i < commodity.getGoodsNum(); i++) {
                EntranceQrcode enQRCode = EntranceQrcode.builder()
                        .associationId(Long.valueOf(commodity.getGoodsId()))
                        .ticketType(commodity.getType())
                        .status(1)
                        .endTime(commodity.getValidity())
                        .userId(order.getUserId()) // 绑定用户id
                        .build();
                entranceQrcodesInsert.add(enQRCode);
            }
        }

        this.saveBatch(entranceQrcodesInsert);

        //为每张票生成二维码
        List<String> QRCodeBytes = new ArrayList<>();
        try {
            QRCodeBytes = produceQRCodeUrl(entranceQrcodesInsert);
        } catch (Exception e) {
            return BaseResponse.error("二维码生成异常", new EntranceQrcode());
        }

        return BaseResponse.ok(QRCodeBytes);
    }


    /**
     * 有票务票据生成二维码
     *
     * @param list 票据集合
     * @return 二维码集合
     */
    private List<String> produceQRCodeUrl(List<EntranceQrcode> list) {
        //为每张票生成二维码
        List<String> QRCodeBytes = new ArrayList<>();
        try {
            for (EntranceQrcode enQRCode : list) {
                String enQCodeJSON = JSON.toJSONString(enQRCode);
                //加密字符串
                String encryStr = encryption(enQCodeJSON);
                QRCodeBytes.add(encryStr);
            }
        } catch (Exception e) {
            throw new RuntimeException("二维码生成失败!");
        }

        return QRCodeBytes;
    }

    @Override
    public BaseResponse scanEntranceQRcode(String enQRCodeStr) {
        String deCodeQRCodeStr = "";
        try {
            deCodeQRCodeStr = deCode(enQRCodeStr);
        } catch (Exception e) {
            return BaseResponse.error("无效票据!", this.getClass().getName());
        }

        assert StrUtil.isNotEmpty(deCodeQRCodeStr);

        EntranceQrcode enQRCode = JSON.parseObject(deCodeQRCodeStr, EntranceQrcode.class);
        QueryWrapper ew = new QueryWrapper();
        ew.eq("QRcode_id", enQRCode.getId());
        ew.eq("user_id", enQRCode.getUserId());
        ew.eq("association_id", enQRCode.getAssociationId());
        ew.eq("ticket_type", enQRCode.getTicketType());
        int count = iEntranceLogService.count(ew);

        if (count == 0) {
            EntranceLog entranceLogIn = EntranceLog.builder()
                    .associationId(enQRCode.getAssociationId())
                    .createTime(System.currentTimeMillis())
                    .qrcodeId(enQRCode.getId())
                    .ticketType(enQRCode.getTicketType())
                    .userId(enQRCode.getUserId())
                    .build();
            iEntranceLogService.save(entranceLogIn);
            return BaseResponse.ok("可以入场");
        }

        if ("SINGLE".equals(enQRCode.getTicketType())) {
            return BaseResponse.error("入场券已被使用", enQRCode);
        } else {
            ew.orderByDesc("create_time");
            List<EntranceLog> list = iEntranceLogService.list(ew);
            EntranceLog entranceLog = list.get(0);
            //判断起始日期是否一致
            if (DateUtil.beginOfDay(new Date(entranceLog.getCreateTime())).compareTo(DateUtil.beginOfDay(new Date(System.currentTimeMillis()))) == 0) {
                return BaseResponse.error("每日只能使用一次", entranceLog);
            } else {
                //如果时间不是今天的 则添加一次今天入场信息
                EntranceLog entranceLogIn = EntranceLog.builder()
                        .associationId(enQRCode.getAssociationId())
                        .createTime(System.currentTimeMillis())
                        .qrcodeId(enQRCode.getId())
                        .ticketType(enQRCode.getTicketType())
                        .userId(enQRCode.getUserId())
                        .build();
                iEntranceLogService.save(entranceLogIn);
                return BaseResponse.ok("可以入场");
            }
        }
    }

    @Override
    public BaseResponse<List<RQCodeInfoVo>> getQRCodeInfoPo(String userId) {

        if (StrUtil.isEmpty(userId)) {
            return BaseResponse.error("用户id为必填!", this.getClass().getName());
        }
        List<RQCodeInfoDo> rqCodeInfoDos = entranceQrcodeMapper.getQRCodeInfoPo(userId);

        //为每张票生成二维码
        List<RQCodeInfoVo> QRCodeBytes = new ArrayList<>();
        try {
            for (RQCodeInfoDo rqCodeInfoDo : rqCodeInfoDos) {
                EntranceQrcode enQRCode = rqCodeInfoDo.getEntranceQrcode();
                String enQCodeJSON = JSON.toJSONString(enQRCode);

                //加密字符串
                String encryStr = encryption(enQCodeJSON);
                RQCodeInfoVo rqCodeInfoVo = new RQCodeInfoVo();
                rqCodeInfoVo.setUrl(encryStr);
                rqCodeInfoVo.setTitle(rqCodeInfoDo.getTitle());
                rqCodeInfoVo.setExUrl(fastDFSUtils.fullUrl(rqCodeInfoDo.getExUrl()));
                rqCodeInfoVo.setDescribe(rqCodeInfoDo.getDescribe());
                QRCodeBytes.add(rqCodeInfoVo);
            }
        } catch (Exception e) {
            log.error("票据生成二维码失败! === cn.wise.sc.pay.business.exhibition.service.impl.EntranceQrcodeServiceImpl.getQRCodeInfoPo");
            return BaseResponse.error("获取用户票据二维码失败!", this.getClass().getName());
        }

        return BaseResponse.ok(QRCodeBytes);
    }

    @Override
    public BaseResponse<EntranceQrcode> shareQRCode(QRCodeGiftQuery qrCodeGiftQuery) {

        //获取票据信息  并解密
        String QRCodeStr = qrCodeGiftQuery.getQRCodeStr();
        if (StrUtil.isEmpty(QRCodeStr)) {
            return BaseResponse.error("票据信息为空!", this.getClass().getName());
        }
        String deCodeQRCodeStr = deCode(QRCodeStr);
        EntranceQrcode entranceQrcode = JSON.parseObject(deCodeQRCodeStr, EntranceQrcode.class);

        EntranceQrcode updateQRCode = new EntranceQrcode();
        updateQRCode.setId(entranceQrcode.getId());
        updateQRCode.setUserId(qrCodeGiftQuery.getNewUserId());
        QueryWrapper<EntranceQrcode> qw = new QueryWrapper<>();
        qw.eq("user_id", entranceQrcode.getUserId());
        qw.eq("id",entranceQrcode.getId());
        boolean update = this.update(updateQRCode, qw);

        return update ? BaseResponse.okMsg("领取成功!", this.getClass()) : BaseResponse.error("领取失败,已被领取!", this.getClass().getName());
    }

    /**
     * 加密
     *
     * @param str 待加密字符串
     * @return 加密后的数据
     */
    private String encryption(String str) {
        //构建
        SymmetricCrypto aes = new SymmetricCrypto(SymmetricAlgorithm.AES, key);
        //加密为16进制表示
        return aes.encryptHex(str);
    }

    /**
     * 解密
     *
     * @param str 待解密的字符串
     * @return 解密后的数据
     */
    private String deCode(String str) {
        //构建
        SymmetricCrypto aes = new SymmetricCrypto(SymmetricAlgorithm.AES, key);
        //解密为字符串
        return aes.decryptStr(str, CharsetUtil.CHARSET_UTF_8);
    }
}
