package cn.wise.sc.pay.business.order.service;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.lang.Snowflake;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;
import cn.wise.sc.pay.business.order.configure.WxPayConfigure;
import cn.wise.sc.pay.business.order.model.cmbnoenet.CMBONeNetResponse;
import cn.wise.sc.pay.business.order.model.cmbnoenet.CMBOneNetOrderParams;
import cn.wise.sc.pay.business.order.model.cmbnoenet.CMBOneNetRequestDataModel;
import cn.wise.sc.pay.business.order.model.cmbnoenet.CMBOneNetRequestModel;
import cn.wise.sc.pay.business.order.model.cmbnoenet.CMBOneNetRequestUrl;
import cn.wise.sc.pay.business.order.model.cmbnoenet.CMBOneNetResponseFlow;
import cn.wise.sc.pay.business.order.model.cmbnoenet.CMBOneNetTokenInfo;
import cn.wise.sc.pay.business.order.utils.AllInOneNetUtil;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import com.alibaba.fastjson.JSON;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpMethod;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;

import java.util.Date;
import java.util.HashMap;
import java.util.Map;


/**
 * 招行一卡通 支付
 */
@Slf4j
@Service
public class CmbOneNetPayService {

    @Autowired
    WxPayConfigure wxPayConfigure;


    /**
     * 解析app端授权信息
     *
     * @param signature Token 报文签名信息，使用开放平台私钥对加密后的数据的哈希值进 行签名。采用 2048 位 RSA 密钥签名时，其长度固定为 344 字节
     * @param data      采用分行小程序公钥进行 RSA 加密后的 Token 报文
     * @return
     */
    public BaseResponse getUserInfo(String signature, String data) {
        if (StrUtil.isEmpty(data) || StrUtil.isEmpty(signature)) {
            return BaseResponse.error("signature和data为必填项!", this.getClass().getName());
        }

        // signature和 data转码
        signature = transcoding(signature);
        data = transcoding(data);

        try {
            String token = AllInOneNetUtil.decrypt(AllInOneNetUtil.priKey, data);
            CMBOneNetTokenInfo cmbOneNetTokenInfo = JSON.parseObject(token, CMBOneNetTokenInfo.class);
            return BaseResponse.ok(cmbOneNetTokenInfo);
        } catch (Exception e) {
            log.error("招行一卡通手令牌解析异常! 参数:{}", data);
            return BaseResponse.error("招行一卡通手令牌解析异常", this.getClass().getName());
        }
    }

    /**
     * 将参数中的 +换成- ,/替换成_
     *
     * @param waitTransStr 带处理字符串
     * @return String
     */
    private String transcoding(String waitTransStr) {
        return waitTransStr.replace("\\+", "-").replace("\\/", "_");
    }

    /**
     * 招行一卡通  下订单
     *
     * @param orderNo     订单号
     * @param txnAmt      金额
     * @param receAccount 接收方账户
     * @param receName    接收方账户名
     * @param receBank    接收方开户银行
     * @return
     */
    public BaseResponse oneNetOrderInfo(
            String orderNo, String txnAmt, String receAccount, String receName, String receBank, String body)
            throws Exception {

        // 封装业务参数
        CMBOneNetOrderParams params = new CMBOneNetOrderParams();
        params.setCcynbr("10");
        params.setMchtyp("MCHTYP004");
        params.setRcvbbk("313234001089");
        params.setRcveac(receAccount);
        params.setReford(orderNo);
        params.setTrsamt(txnAmt);
        params.setRcvean(receName);
        params.setDetinf(body);
        Map<String, String> bodyMap = new HashMap<>();
        bodyMap.put("orderinfo", JSON.toJSONString(params));

        // 请求订单信息
        CMBONeNetResponse cmboNeNetResponse =
                postCMBApi(bodyMap, CMBOneNetRequestUrl.ORDERINFO_URL, "");

        // 处理响应结果
        if (cmboNeNetResponse != null) {
            if ("SUC0000".equals(cmboNeNetResponse.getResponse().getHead().getRetcod())) {
                Map<String, String> result = new HashMap(3);
                // 解密实例流程号
                CMBOneNetResponseFlow cmbOneNetResponseFlow =
                        decodeFlowNo(cmboNeNetResponse.getResponse().getBody());
                String flow = cmbOneNetResponseFlow.getSubresult().getSeqnbr();
                String merId = cmboNeNetResponse.getResponse().getHead().getTppidc();
                String time = cmboNeNetResponse.getResponse().getHead().getRsptim();
                String sigdat = sigdat(flow, time, time);
                String srcdat = srcdat(flow, merId, time);
                result.put("sigdat", sigdat);
                result.put("srcdat", srcdat);
                result.put("merId", merId);
                return BaseResponse.ok(result);
            }
            return BaseResponse.error(cmboNeNetResponse.getResponse().getHead().getRetmsg(), "招行一卡通支付");
        } else {
            return BaseResponse.error("唤起招行一卡通失败", "招行一卡通支付");
        }
    }

    /**
     * 获取Api访问令牌
     *
     * @param authCode 授权码
     * @return acctkn
     */
    public BaseResponse getApplyInfo(String authCode) {
        if (StrUtil.isEmpty(authCode)) {
            return BaseResponse.error("授权码是必填项", this.getClass().getName());
        }
        HashMap<String, String> bodyMap = new HashMap<>(1);
        bodyMap.put("applyinfo", "{\"autcod:\"" + authCode + "}");
        CMBONeNetResponse cmboNeNetResponse = null;
        try {
            cmboNeNetResponse = postCMBApi(bodyMap, CMBOneNetRequestUrl.APPLYINFO_URL, "FOATKAPL");
        } catch (Exception e) {
            log.error("发起获取Api访问令牌的请求失败! 参数:{}", JSON.toJSONString(bodyMap));
        }
        // 处理响应
        if (cmboNeNetResponse != null) {
            if ("SUC0000".equals(cmboNeNetResponse.getResponse().getHead().getRetcod())) {
                String response = null;
                try {
                    response =
                            AllInOneNetUtil.decrypt(
                                    AllInOneNetUtil.priKey, cmboNeNetResponse.getResponse().getBody());
                } catch (Exception e) {
                    log.error("解析获取访问令牌响应失败!参数:{}", response);
                    return BaseResponse.error("获取访问令牌失败!", this.getClass().getName());
                }
                return BaseResponse.ok(response);
            }
            return BaseResponse.error(cmboNeNetResponse.getResponse().getHead().getRetmsg(), this.getClass().getName());
        } else {
            return BaseResponse.error("获取访问令牌失败!", this.getClass().getName());
        }
    }

    /**
     * 续费访问token
     * 1.过期2h不得续费
     * 2.续费次数超过4次 不得续费
     *
     * @param token 带续费的token
     * @return BaseResponse
     */
    public BaseResponse renewApplyInfo(String token) {
        if (StrUtil.isEmpty(token)) {
            return BaseResponse.error("token参数是必填的!", this.getClass().getName());
        }
        HashMap<String, String> bodyMap = new HashMap<>(1);
        bodyMap.put("renewinfo", "{\"autcod:\"" + token + "}");
        CMBONeNetResponse cmboNeNetResponse = null;
        try {
            cmboNeNetResponse = postCMBApi(bodyMap, CMBOneNetRequestUrl.RENEWINFO_URL, "FOATKAPL");
        } catch (Exception e) {
            log.error("续费Api访问令牌的请求失败! 参数:{}", JSON.toJSONString(bodyMap));
        }
        // 处理响应
        if (cmboNeNetResponse != null) {
            if ("SUC0000".equals(cmboNeNetResponse.getResponse().getHead().getRetcod())) {
                String response = null;
                try {
                    response =
                            AllInOneNetUtil.decrypt(
                                    AllInOneNetUtil.priKey, cmboNeNetResponse.getResponse().getBody());
                } catch (Exception e) {
                    log.error("续费Api访问令牌的请求失败!参数:{}", response);
                    return BaseResponse.error("续费Api访问令牌的请求失败!", this.getClass().getName());
                }
                return BaseResponse.ok(response);
            }
            return BaseResponse.error(cmboNeNetResponse.getResponse().getHead().getRetmsg(), this.getClass().getName());
        } else {
            return BaseResponse.error("续费Api访问令牌的请求失败!", this.getClass().getName());
        }
    }

    /**
     * 解密流程号
     *
     * @param flow 流程加密字段
     * @return
     */
    private CMBOneNetResponseFlow decodeFlowNo(String flow) {
        if (StrUtil.isEmpty(flow)) {
            throw new IllegalArgumentException("招行一卡通----流程号为空!");
        }
        try {
            String decryptFlow = AllInOneNetUtil.decrypt(AllInOneNetUtil.priKey, flow);
            return JSON.parseObject(decryptFlow, CMBOneNetResponseFlow.class);
        } catch (Exception e) {
            throw new RuntimeException("招行一卡通支付----解密实例流程号,异常！");
        }
    }

    /**
     * 获取唯一标识
     *
     * @return
     */
    private String getNewMsgidc() {
        Snowflake snowflake = IdUtil.createSnowflake(1, 1);
        return snowflake.nextIdStr();
    }

    /**
     * 逗号分隔，需要先加密、后签名。 流程实例号用于查订单
     *
     * @param seqnbr 流程实例号
     * @param merId  商户编号
     * @param time   时间 戳（格式为 yyyyMMddHHmmss
     * @return
     */
    private String sigdat(String seqnbr, String merId, String time) {
        return null;
    }

    /**
     * 逗号分隔，需要加密。流程实例号 用于查订单
     *
     * @param seqnbr 流程实例号
     * @param merId  商户编号
     * @param time   时间 戳（格式为 yyyyMMddHHmmss
     * @return
     */
    private String srcdat(String seqnbr, String merId, String time) {
        return null;
    }

    /**
     * 请求招行api
     *
     * @param bodyMap 请求体
     * @param url     api路径
     * @param trscod  类型 招行的trscod
     * @return ResponseEntity的body
     */
    private CMBONeNetResponse postCMBApi(Map<String, String> bodyMap, String url, String trscod)
            throws Exception {
        String body = JSON.toJSONString(bodyMap);
        System.out.println("==================body===================");
        System.out.println(body);

        // 加密业务参数
        String bodyEncrypt = AllInOneNetUtil.encrypt(AllInOneNetUtil.pubKey, body, "utf-8");

        // 设置请求体中得head字段
        CMBOneNetRequestDataModel requestDataModel = new CMBOneNetRequestDataModel();
        requestDataModel.setBody(bodyEncrypt);
        CMBOneNetRequestDataModel.Header header = requestDataModel.new Header();
        // 商户号
//        header.setTppidc(wxPayConfigure.CMB_ONENET_MERID);
        header.setTppidc("B000000168");
        // 设置请求类型
        if (StrUtil.isNotEmpty(trscod)) {
            header.setTrscod(trscod);
        } else {
            header.setTrscod("FMORDSUB");
        }
        // 报文唯一标识
        header.setMsgidc(getNewMsgidc());
        String now = DateUtil.format(new Date(), "yyyyMMddHHmmss");
        header.setReqtim(now);

        requestDataModel.setHead(header);
        System.out.println("=====================数据域========================");
        System.out.println(JSON.toJSONString(requestDataModel));

        String sign = AllInOneNetUtil.sign(AllInOneNetUtil.priKey, JSON.toJSONString(requestDataModel), "utf-8");
        CMBOneNetRequestModel requestModel = new CMBOneNetRequestModel();
        requestModel.setRequest(requestDataModel);
        requestModel.setSignature(sign);

        HttpEntity<String> requestEntity =
                new HttpEntity<String>(JSON.toJSONString(requestModel), new HttpHeaders());
        RestTemplate template = new RestTemplate();

        ResponseEntity<String> response =
                template.exchange(url, HttpMethod.POST, requestEntity, String.class);
        if (response.getStatusCode().value() == 200) {
            String rst = response.getBody();
            return JSON.parseObject(rst, CMBONeNetResponse.class);
        }
        return null;
    }

    /**
     * 获取用户手机号等信息
     *
     * @param userId 用户id
     * @param token  访问token
     * @return
     */
    public BaseResponse getUserPhone(String userId, String token) {

        if (StrUtil.isEmpty(token) || StrUtil.isEmpty(userId)) {
            return BaseResponse.error("token&userId参数是必填的!", this.getClass().getName());
        }
        HashMap<String, String> bodyMap = new HashMap<>(1);
        bodyMap.put("usrqryitf ", "{\"acctkn :\"" + token + ",\"usridc:\"" + userId);
        CMBONeNetResponse cmboNeNetResponse = null;
        try {
            cmboNeNetResponse = postCMBApi(bodyMap, CMBOneNetRequestUrl.USERINFO_URL, "FOATKAPL");
        } catch (Exception e) {
            log.error("续费Api访问令牌的请求失败! 参数:{}", JSON.toJSONString(bodyMap));
        }
        // 处理响应
        if (cmboNeNetResponse != null) {
            if ("SUC0000".equals(cmboNeNetResponse.getResponse().getHead().getRetcod())) {
                String response = null;
                try {
                    response =
                            AllInOneNetUtil.decrypt(
                                    AllInOneNetUtil.priKey, cmboNeNetResponse.getResponse().getBody());
                } catch (Exception e) {
                    log.error("续费Api访问令牌的请求失败!参数:{}", response);
                    return BaseResponse.error("续费Api访问令牌的请求失败!", this.getClass().getName());
                }
                return BaseResponse.ok(response);
            }
            return BaseResponse.error(cmboNeNetResponse.getResponse().getHead().getRetmsg(), this.getClass().getName());
        } else {
            return BaseResponse.error("续费Api访问令牌的请求失败!", this.getClass().getName());
        }
    }
}
